# -*- coding: utf-8 -*-
#
# Copyright © 2008 Ignacio Vazquez-Abrams All rights reserved.
# Copyright © 2008 Red Hat, Inc. All rights reserved.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.  You should have
# received a copy of the GNU General Public License along with this program;
# if not, write to the Free Software Foundation, Inc., 51 Franklin Street,
# Fifth Floor, Boston, MA 02110-1301, USA. Any Red Hat trademarks that are
# incorporated in the source code or documentation are not subject to the GNU
# General Public License and may only be used or replicated with the express
# permission of Red Hat, Inc.
#
# Author(s): Ignacio Vazquez-Abrams <ivazquez@fedoraproject.org>

import turbogears.database as database
import sqlalchemy as sa
from fedora.tg.json import SABase
from fas.model.fasmodel import PeopleTable, PersonRolesTable, GroupsTable

database.get_engine()

PlanetTable = sa.Table('planets', database.metadata, autoload=True)

ValidEntries = sa.select(
    [PeopleTable.c.username, PlanetTable.c.id, PlanetTable.c.name,
        PlanetTable.c.feed, PlanetTable.c.hackergotchi],
    from_obj=[PeopleTable.join(
            PlanetTable, PeopleTable.c.id == PlanetTable.c.person_id
        ).join(
            PersonRolesTable, PlanetTable.c.person_id == PersonRolesTable.c.person_id
        ).join(
            GroupsTable, PersonRolesTable.c.group_id == GroupsTable.c.id
        )]
).where(
    sa.and_(
        GroupsTable.c.name == 'fedoraplanet',
        PersonRolesTable.c.role_status == 'approved'
    )
)

PlanetGroupEntries = sa.select(
    [PersonRolesTable.c.person_id],
    from_obj=[PersonRolesTable.join(
        GroupsTable, sa.and_(
            PersonRolesTable.c.group_id == GroupsTable.c.id,
            GroupsTable.c.name == 'fedoraplanet',
            PersonRolesTable.c.role_status == 'approved'
        )
    )]
)

PlanetGroupEntriesAlias = PlanetGroupEntries.alias('planetgroup')

InvalidEntries = sa.select(
    [PlanetTable.c.id,
        PeopleTable.c.username,
        PlanetTable.c.name,
        PlanetTable.c.feed,
        PlanetTable.c.hackergotchi],
).select_from(PlanetTable.join(
        PeopleTable, PlanetTable.c.person_id == PeopleTable.c.id
    ).outerjoin(
        PlanetGroupEntriesAlias, PlanetTable.c.person_id == PlanetGroupEntriesAlias.c.person_id
    )
).where(
    PlanetGroupEntriesAlias.c.person_id == None
)

class Planets(SABase):
    '''All Planet entries in the database'''
    
    @classmethod
    def by_id(cls, id):
        '''Returns an entry with a specifc ID'''
        return cls.query.filter_by(id=id).one()
        
    @classmethod
    def by_personid(cls, id):
        '''Returns all entries belonging to a single person ID'''
        return cls.query.filter_by(person_id=id)
        
    @classmethod
    def by_person(cls, person):
        '''Returns all entries belonging to a single Person'''
        return cls.query.filter_by(person_id=person.id)

database.mapper(Planets, PlanetTable)
